function [img]=bsm_proposeNewImage(img,t)
% function [img]=proposeNewImage(img)
% Propose a new image for the simulated annealing 
% Last updated 9/7 by Erik Wernersson

%img=originalProposeNewImage(img,t);

% This is a homogenity assumption
%if rand(1,1)>0.99
%    img=fillHoles(img);
%end
%img=homogenity(img,t);
%img=homogenity2(img,t);
%img=brown(img,t);

img=flip(img); % The most simple, just flip a pixel
end

function [img]=originalProposeNewImage(img,t)
max_changes=ceil(t^8); % The maximum of allowed changes at one step.
% the minimum will automatically be 4.

nl=numel(img);
for i = 1:round(4+rand(max_changes))
    % interchange two elements    
    a = round(rand(1,1)*(nl-1))+1;
    b = round(rand(1,1)*(nl-1))+1;
    t1= img(a);
    t2= img(b);
    if t1==t2 % Make sure that elements are changed
        i = i-1;
    else
        img(a)=t2;
        img(b)=t1;
    end
end
end

function [img]=homogenity(img,t)
% not all conditions can be guaranteed to be fulfilled
max_changes=ceil(t^8); % move this amout of pixels
p=sqrt(numel(img)); % get the number of elements at the side
img2=reshape(img,p,p); % get the side length

for i = 1:round(1+rand(max_changes))

    q=1; % The q is required to avoid that no pixels fulfill the critera 
while q<100 % find a loose pixel
    q=q+1;
a=round(rand(1,1)*(p-1))+1;
b=round(rand(1,1)*(p-1))+1;
% use 4 connectivity
if img2(a,b)==1 %there must be something there
if fourHood(a,b,img2,p) < 2
    break
end
end
end

q=1;
while q<100 % find a place to put it
    q=q+1;
 c=round(rand(1,1)*(p-1))+1;
 d=round(rand(1,1)*(p-1))+1;
 if img2(c,d)==0 %this should be empty
 if fourHood(c,d,img2,p)>2
     break
 end
 end
end
% change position of pixels
img2(a,b)=0;
img2(c,d)=1;
end

img=img2(:); % rehape the image to a column

end

function [img]=homogenity2(img,t)


end

function [img]=fillHoles(img)
p=sqrt(numel(img));
img = reshape(img,p,p);
c=0; % number of filled holes
for i = 1:p
    for j = 1:p
        if img(i,j)==0
            if fourHood(i,j,img,p)==4
                img(i,j)=1;
                c=c+1;
            end
    end
end
end

if c > 0 
            disp(strcat(int2str(c), '-holes removed'));
end

while c > 0 % remove as many pixels to keep the balance

    % pick a random pixel
% if something is there and is quite loose, remove it.
a=round(rand(1,1)*(p-1))+1;
b=round(rand(1,1)*(p-1))+1;
    if img(a,b)==1
        if fourHood(a,b,img,p)<3
            img(a,b)=0;
            c=c-1;
        end
    end
end

    img=img(:);


end

function npix=fourHood(a,b,img,res)
% returns the numer of non-zeros pixels in the four nighbourhood of img
n=isPixel(a+1,b,img, res);
s=isPixel(a-1,b,img, res);
e=isPixel(a,b+1,img, res);
w=isPixel(a,b-1,img, res);
npix=n+s+e+w;
end

function ret = isPixel(y,x,img,res)
ret=0;
if y > 0 && y < res+1
    if x > 0 && x < res+1
       ret= img(y,x);
    end
end
end

function [img]=brown(img,t)
p=sqrt(numel(img));
img = reshape(img,p,p); % reshape into quadratic again

n_pixels=2; % number of pixels to try to move
steps=30; % number of diffusion steps


% pick a random pixel
% if something is there and is quite loose, remove it.
for i = 1:n_pixels
a=round(rand(1,1)*(p-1))+1;
b=round(rand(1,1)*(p-1))+1;

    if img(a,b)==1 % if there is something at that position
        if fourHood(a,b,img,p) < 4 % if it has a non-zero neighbourhood
            img=diffuse(a,b,img,steps,p);
        end        
    end
end
% reshape and return the image
    img=img(:);
end

function img=diffuse(a,b,img,steps,p)
%Diffuse the pixel at a,b steps in a quadratic image(img) with resolution p
%x p

for k = 1:steps
dir=ceil(4*rand(1,1)); % pick a direction randomly

if dir==1
    mv=[1 0];
  % move upwards 
end

if dir==2
    mv=[-1 0];
  % move upwards 
end

if dir==3
    mv=[0 1];
  % move upwards 
end

if dir==4
    mv=[0 -1];
  % move upwards 
end

% Calculate the new coordinate
c=a+mv(1);
d=b+mv(2);

if moveHereOk(c,d,img,p)==1 %within image and free
    img(a,b)=0; % original position is empty
    img(c,d)=1; % new position is filled
    a=c; b=d;
end

end

end

function ret=moveHereOk(y,x,img,res)
% y,x coordinates in img: image with side length: p
% returns 1 if it is ok to diffuse here
%
ret=0;
if y > 0 && y < res+1
    if x > 0 && x < res+1
       ret= img(y,x)+1;
    end
end

end

function img=flip(img)
[nl]=size(img);
a = round(rand(1,1)*(nl-1))+1;

if img(a)==1;
    img(a)=0;
else
    img(a)=1;
end

end